#!/bin/sh
exec isabelle scala "$0" "$@"
!#

import isabelle._
import scala.io.Source

def read_times (sessions:List[String]):scala.collection.mutable.Map[Tuple2[String,String], String] = {
  val user_output_dir: Path = Path.explode("$ISABELLE_HEAPS/$ML_IDENTIFIER")
  val store = Sessions.store(Options.init())
  var acc = scala.collection.mutable.Map[Tuple2[String,String], String]()
  for (session <- sessions) {
    val fq_session = "PSPSP-"+session
    val db_path = user_output_dir+store.database(fq_session)
    if (db_path.file.exists()) {
      val db = SQLite.open_database(db_path)
      val command_timings = store.read_command_timings(db, fq_session)
      for (t <- command_timings) {
        val (_, name) = t.filter(e => e._1 == "name").head
        val (_, time) = t.filter(e => e._1 == "elapsed").head
        acc += ((session,name) -> time)
      }
      val theory_timings = store.read_theory_timings(db, fq_session)
      if (theory_timings.isEmpty) {
      }else{    
        val (_, time) = theory_timings.head.filter(e => e._1 == "elapsed").head
        acc += ((session,"theory") -> time)
      }
    }
  }
  return acc
}


def lookup(map:scala.collection.mutable.Map[Tuple2[String,String],String], key:Tuple2[String,String]) = {
  if(map contains key){
    s""""${map(key)}""""
  }else{
    ""
  }
}

def mk_examples(sessions:List[String]):List[String]= {
    sessions.map(e => if(e.endsWith("_safe")){
                     e.dropRight(5)
                 }else{
                     if(e.endsWith("_unsafe")){
                           e.dropRight(7)
                       }else{ // _nbe
                              e.dropRight(4)
                            }
                 }).distinct
}

def print_csv_t (sessions:List[String],times:scala.collection.mutable.Map[Tuple2[String,String], String]) = {
  val examples = mk_examples(sessions)
  // Header
  print(s""""Protocol"""")
  for (ex_type <- ex_types) {
     for (timing_type <- timing_types) {
      print(s""", "${timing_type._2} [${ex_type}]"""")
    }
  }
  println("")
  
  // Body
  
  for (ex <- examples){
    print(s""""$ex"""")
    for (ex_type <- ex_types) {
      for (timing_type <- timing_types) {
        print(s""", ${lookup(times,(ex+"_"+ex_type,timing_type._1))}""")
      }
    }
    println("")
  }
}

def print_csv (sessions:List[String],times:scala.collection.mutable.Map[Tuple2[String,String], String]) = {
  val examples = mk_examples(sessions)
                                    
  // Header
  for (ex_name <- examples) {
    for(ex_type <- ex_types) {
      print(s""", "${ex_name+"_"+ex_type}"""")
    }
  }
  println("")        
                 
  // Body 
  for (timing_type <- timing_types) {
    print(s""""${timing_type._2}"""")
    for (ex_name <- examples) {
      for (ex_type <- ex_types){
           print(s""", ${lookup(times,(ex_name+"_"+ex_type,timing_type._1))}""")
      }
    }
    println("")
  }
}
  
def main(args: Array[String]) {
val usage = """
    Usage: isabelle scala extract_timing_as_csv.scala [--help] [--transpose] [--session] ROOTS_FILE_OR_SESSION_NAME

           --help       prints this help message
           --transpose  prints table in alternative ("transposed") format
           --session    selects a single session, i.e., ROOTS_FILE_OR_SESSION_NAME should be the name of a 
                        session. Without this options, all sessions in the given ROOTS file are processed
           [ROOTS_FILE_OR_SESSION_NAME] depending on option "--session", this is either the name of a session
                                        or the path of a ROOTS file
  """
  val arglist = args.toList
  type OptionMap = Map[Symbol, Any]
  
  def nextOption(map : OptionMap, list: List[String]) : OptionMap = {
    def isSwitch(s : String) = (s(0) == '-')
      list match {
        case Nil => map
        case "--transpose" :: tail => nextOption(map ++ Map('transpose -> true), tail)
        case "--session" :: tail   => nextOption(map ++ Map('session -> true), tail)
        case "--help" :: tail      => println(usage) 
                                      System.exit(1)
                                      map
        case string :: opt2 :: tail if isSwitch(opt2) => 
                                      nextOption(map ++ Map('infile -> string), list.tail)
        case string :: Nil         => nextOption(map ++ Map('infile -> string), list.tail)
        case option :: tail        => println("Unknown option "+option) 
                                      System.exit(1)
                                      map
      }
  }
  val options = nextOption(Map(),arglist)

  val sessions = if (options.exists(_ == 'session -> true)) {
    val session = if (options.contains('infile)){
      options('infile).asInstanceOf[String]
    }else{
      "PSPSP-Keyserver_safe"
    }
    var is_full_name = false
    for (postfix <- ex_types){
      is_full_name = is_full_name || session.endsWith(postfix)
    } 
    if (is_full_name){
      List(session)
    }else{
      ex_types.map(postfix => session+"_"+postfix)
    }
  }else{
    val filename = if (options.contains('infile)){
      options('infile).asInstanceOf[String]
    }else{
      "ROOTS"
    }
    Source.fromFile(filename).getLines.toList
  }

  val times = read_times (sessions)
  if (options.exists(_ == 'transpose -> true)) {
    print_csv_t(sessions,times)
  }else{
    print_csv(sessions,times)
  }

}


val timing_types: List[Tuple2[String,String]] = List(("trac", "Encoding trac"),
                                                     ("protocol_model_setup","Setup Protocol Model"),
                                                     ("compute_fixpoint","Compute Fixed Point"),
                                                     ("protocol_security_proof","Security Proof"),
                                                     ("theory","Total Theory Processing")
                                                   )
val ex_types: List[String] = List("safe", "nbe", "unsafe")


main(args)
